"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// packages/@code-like-a-carpenter/foundation-runtime/src/functions/enricher/make-enricher.ts
var make_enricher_exports = {};
__export(make_enricher_exports, {
  Enricher: () => Enricher,
  makeEnricher: () => makeEnricher
});
module.exports = __toCommonJS(make_enricher_exports);
var import_assert = require("@code-like-a-carpenter/assert");
var import_telemetry = require("@code-like-a-carpenter/telemetry");
var import_errors = require("../../errors/index.cjs");
var import_cdc_handler = require("../common/cdc-handler.cjs");
var import_handlers = require("../common/handlers.cjs");
function makeEnricher(constructor, sdk, dependencies) {
  const enricher = new constructor(sdk);
  return (0, import_handlers.makeSqsHandler)(async (unmarshalledRecord) => {
    await enricher.enrich(unmarshalledRecord);
  }, dependencies);
}
var Enricher = class extends import_cdc_handler.CDCHandler {
  sdk;
  constructor(sdk) {
    super();
    this.sdk = sdk;
  }
  async enrich(unmarshalledRecord) {
    return this.runWithNewSpan("enrich", async () => {
      const { createTargetModel, updateTargetModel, unmarshallSourceModel } = this.sdk;
      (0, import_assert.assert)(
        unmarshalledRecord.dynamodb?.NewImage,
        "NewImage missing from DynamoDB Stream Event. This should never happen."
      );
      const source = unmarshallSourceModel(
        unmarshalledRecord.dynamodb?.NewImage
      );
      const previous = unmarshalledRecord.dynamodb?.OldImage && unmarshallSourceModel(unmarshalledRecord.dynamodb.OldImage);
      (0, import_telemetry.getCurrentSpan)()?.setAttributes({
        "com.code-like-a-carpenter.foundation.previous.exists": !!previous
      });
      try {
        const target = await this.runWithNewSpan(
          "load",
          () => this.load(source, previous)
        );
        (0, import_telemetry.getCurrentSpan)()?.setAttributes({
          "com.code-like-a-carpenter.foundation.target.exists": !!target
        });
        const modelToUpdate = await this.runWithNewSpan(
          "update",
          () => this.update(source, target, previous)
        );
        (0, import_telemetry.getCurrentSpan)()?.setAttributes({
          "com.code-like-a-carpenter.foundation.should-update": !!modelToUpdate
        });
        if (modelToUpdate) {
          await this.runWithNewSpan(
            "updateTargetModel",
            () => updateTargetModel(modelToUpdate)
          );
        }
        return;
      } catch (err) {
        if (err instanceof import_errors.NotFoundError) {
          const modelToCreate = await this.runWithNewSpan(
            "create",
            () => this.create(source, previous)
          );
          (0, import_telemetry.getCurrentSpan)()?.setAttributes({
            "com.code-like-a-carpenter.foundation.should-create": !!modelToCreate
          });
          if (modelToCreate) {
            await this.runWithNewSpan(
              "createTargetModel",
              () => createTargetModel(modelToCreate)
            );
          }
          return;
        }
        throw err;
      }
    });
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  Enricher,
  makeEnricher
});
//# sourceMappingURL=make-enricher.cjs.map
