"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// packages/@code-like-a-carpenter/foundation-runtime/src/functions/multi-reducer/make-multi-reducer.ts
var make_multi_reducer_exports = {};
__export(make_multi_reducer_exports, {
  MultiReducer: () => MultiReducer,
  makeMultiReducer: () => makeMultiReducer
});
module.exports = __toCommonJS(make_multi_reducer_exports);
var import_assert = require("@code-like-a-carpenter/assert");
var import_telemetry = require("@code-like-a-carpenter/telemetry");
var import_cdc_handler = require("../common/cdc-handler.cjs");
var import_handlers = require("../common/handlers.cjs");
function makeMultiReducer(constructor, sdk, dependencies) {
  const reducer = new constructor(sdk);
  return (0, import_handlers.makeSqsHandler)(async (record) => {
    await reducer.reduce(record);
  }, dependencies);
}
var MultiReducer = class extends import_cdc_handler.CDCHandler {
  sdk;
  constructor(sdk) {
    super();
    this.sdk = sdk;
  }
  async reduce(unmarshalledRecord) {
    return this.runWithNewSpan("multi-reduce", async () => {
      const { createTargetModel, updateTargetModel, unmarshallSourceModel } = this.sdk;
      (0, import_assert.assert)(
        unmarshalledRecord.dynamodb?.NewImage,
        "NewImage missing from DynamoDB Stream Event. This should never happen."
      );
      const source = unmarshallSourceModel(
        unmarshalledRecord.dynamodb?.NewImage
      );
      const previous = unmarshalledRecord.dynamodb?.OldImage && unmarshallSourceModel(unmarshalledRecord.dynamodb.OldImage);
      (0, import_telemetry.getCurrentSpan)()?.setAttributes({
        "com.code-like-a-carpenter.foundation.previous.exists": !!previous
      });
      const sources = await this.runWithNewSpan(
        "loadSources",
        () => this.loadSources(source, previous)
      );
      (0, import_telemetry.getCurrentSpan)()?.setAttributes({
        "com.code-like-a-carpenter.foundation.sources.count": sources.length
      });
      const targets = await this.runWithNewSpan(
        "loadTargets",
        () => this.loadTargets(source, previous)
      );
      (0, import_telemetry.getCurrentSpan)()?.setAttributes({
        "com.code-like-a-carpenter.foundation.targets.count": targets.length
      });
      const models = await this.runWithNewSpan(
        "createOrUpdate",
        () => this.createOrUpdate(source, sources, targets, previous)
      );
      const modelsToCreate = models.filter(
        (model) => !("version" in model)
      );
      const modelsToUpdate = models.filter(
        (model) => "version" in model
      );
      (0, import_telemetry.getCurrentSpan)()?.setAttributes({
        "com.code-like-a-carpenter.foundation.targets-to-create.count": modelsToCreate.length,
        "com.code-like-a-carpenter.foundation.targets-to-update.count": modelsToUpdate.length
      });
      await Promise.all([
        Promise.all(
          modelsToCreate.map(
            (model) => this.runWithNewSpan(
              "createTargetModel",
              () => createTargetModel(model)
            )
          )
        ),
        Promise.all(
          modelsToUpdate.map(
            (model) => this.runWithNewSpan(
              "updateTargetModel",
              () => updateTargetModel(model)
            )
          )
        )
      ]);
    });
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  MultiReducer,
  makeMultiReducer
});
//# sourceMappingURL=make-multi-reducer.cjs.map
