// packages/@code-like-a-carpenter/foundation-runtime/src/functions/enricher/make-enricher.ts
import { assert } from "@code-like-a-carpenter/assert";
import { getCurrentSpan } from "@code-like-a-carpenter/telemetry";
import { NotFoundError } from "../../errors/index.mjs";
import { CDCHandler } from "../common/cdc-handler.mjs";
import { makeSqsHandler } from "../common/handlers.mjs";
function makeEnricher(constructor, sdk, dependencies) {
  const enricher = new constructor(sdk);
  return makeSqsHandler(async (unmarshalledRecord) => {
    await enricher.enrich(unmarshalledRecord);
  }, dependencies);
}
var Enricher = class extends CDCHandler {
  sdk;
  constructor(sdk) {
    super();
    this.sdk = sdk;
  }
  async enrich(unmarshalledRecord) {
    return this.runWithNewSpan("enrich", async () => {
      const { createTargetModel, updateTargetModel, unmarshallSourceModel } = this.sdk;
      assert(
        unmarshalledRecord.dynamodb?.NewImage,
        "NewImage missing from DynamoDB Stream Event. This should never happen."
      );
      const source = unmarshallSourceModel(
        unmarshalledRecord.dynamodb?.NewImage
      );
      const previous = unmarshalledRecord.dynamodb?.OldImage && unmarshallSourceModel(unmarshalledRecord.dynamodb.OldImage);
      getCurrentSpan()?.setAttributes({
        "com.code-like-a-carpenter.foundation.previous.exists": !!previous
      });
      try {
        const target = await this.runWithNewSpan(
          "load",
          () => this.load(source, previous)
        );
        getCurrentSpan()?.setAttributes({
          "com.code-like-a-carpenter.foundation.target.exists": !!target
        });
        const modelToUpdate = await this.runWithNewSpan(
          "update",
          () => this.update(source, target, previous)
        );
        getCurrentSpan()?.setAttributes({
          "com.code-like-a-carpenter.foundation.should-update": !!modelToUpdate
        });
        if (modelToUpdate) {
          await this.runWithNewSpan(
            "updateTargetModel",
            () => updateTargetModel(modelToUpdate)
          );
        }
        return;
      } catch (err) {
        if (err instanceof NotFoundError) {
          const modelToCreate = await this.runWithNewSpan(
            "create",
            () => this.create(source, previous)
          );
          getCurrentSpan()?.setAttributes({
            "com.code-like-a-carpenter.foundation.should-create": !!modelToCreate
          });
          if (modelToCreate) {
            await this.runWithNewSpan(
              "createTargetModel",
              () => createTargetModel(modelToCreate)
            );
          }
          return;
        }
        throw err;
      }
    });
  }
};
export {
  Enricher,
  makeEnricher
};
//# sourceMappingURL=make-enricher.mjs.map
