// packages/@code-like-a-carpenter/foundation-runtime/src/functions/multi-reducer/make-multi-reducer.ts
import { assert } from "@code-like-a-carpenter/assert";
import { getCurrentSpan } from "@code-like-a-carpenter/telemetry";
import { CDCHandler } from "../common/cdc-handler.mjs";
import { makeSqsHandler } from "../common/handlers.mjs";
function makeMultiReducer(constructor, sdk, dependencies) {
  const reducer = new constructor(sdk);
  return makeSqsHandler(async (record) => {
    await reducer.reduce(record);
  }, dependencies);
}
var MultiReducer = class extends CDCHandler {
  sdk;
  constructor(sdk) {
    super();
    this.sdk = sdk;
  }
  async reduce(unmarshalledRecord) {
    return this.runWithNewSpan("multi-reduce", async () => {
      const { createTargetModel, updateTargetModel, unmarshallSourceModel } = this.sdk;
      assert(
        unmarshalledRecord.dynamodb?.NewImage,
        "NewImage missing from DynamoDB Stream Event. This should never happen."
      );
      const source = unmarshallSourceModel(
        unmarshalledRecord.dynamodb?.NewImage
      );
      const previous = unmarshalledRecord.dynamodb?.OldImage && unmarshallSourceModel(unmarshalledRecord.dynamodb.OldImage);
      getCurrentSpan()?.setAttributes({
        "com.code-like-a-carpenter.foundation.previous.exists": !!previous
      });
      const sources = await this.runWithNewSpan(
        "loadSources",
        () => this.loadSources(source, previous)
      );
      getCurrentSpan()?.setAttributes({
        "com.code-like-a-carpenter.foundation.sources.count": sources.length
      });
      const targets = await this.runWithNewSpan(
        "loadTargets",
        () => this.loadTargets(source, previous)
      );
      getCurrentSpan()?.setAttributes({
        "com.code-like-a-carpenter.foundation.targets.count": targets.length
      });
      const models = await this.runWithNewSpan(
        "createOrUpdate",
        () => this.createOrUpdate(source, sources, targets, previous)
      );
      const modelsToCreate = models.filter(
        (model) => !("version" in model)
      );
      const modelsToUpdate = models.filter(
        (model) => "version" in model
      );
      getCurrentSpan()?.setAttributes({
        "com.code-like-a-carpenter.foundation.targets-to-create.count": modelsToCreate.length,
        "com.code-like-a-carpenter.foundation.targets-to-update.count": modelsToUpdate.length
      });
      await Promise.all([
        Promise.all(
          modelsToCreate.map(
            (model) => this.runWithNewSpan(
              "createTargetModel",
              () => createTargetModel(model)
            )
          )
        ),
        Promise.all(
          modelsToUpdate.map(
            (model) => this.runWithNewSpan(
              "updateTargetModel",
              () => updateTargetModel(model)
            )
          )
        )
      ]);
    });
  }
};
export {
  MultiReducer,
  makeMultiReducer
};
//# sourceMappingURL=make-multi-reducer.mjs.map
