// packages/@code-like-a-carpenter/foundation-runtime/src/functions/reducer/make-reducer.ts
import { assert } from "@code-like-a-carpenter/assert";
import { getCurrentSpan } from "@code-like-a-carpenter/telemetry";
import { NotFoundError } from "../../errors/index.mjs";
import { CDCHandler } from "../common/cdc-handler.mjs";
import { makeSqsHandler } from "../common/handlers.mjs";
function makeReducer(constructor, sdk, dependencies) {
  const reducer = new constructor(sdk);
  return makeSqsHandler(async (record) => {
    await reducer.reduce(record);
  }, dependencies);
}
var Reducer = class extends CDCHandler {
  sdk;
  constructor(sdk) {
    super();
    this.sdk = sdk;
  }
  async reduce(unmarshalledRecord) {
    return this.runWithNewSpan("reduce", async () => {
      const { createTargetModel, updateTargetModel, unmarshallSourceModel } = this.sdk;
      assert(
        unmarshalledRecord.dynamodb?.NewImage,
        "NewImage missing from DynamoDB Stream Event. This should never happen."
      );
      const source = unmarshallSourceModel(
        unmarshalledRecord.dynamodb?.NewImage
      );
      const previous = unmarshalledRecord.dynamodb?.OldImage && unmarshallSourceModel(unmarshalledRecord.dynamodb.OldImage);
      getCurrentSpan()?.setAttributes({
        "com.code-like-a-carpenter.foundation.previous.exists": !!previous
      });
      const sources = await this.runWithNewSpan(
        "loadSources",
        () => this.loadSources(source, previous)
      );
      getCurrentSpan()?.setAttributes({
        "com.code-like-a-carpenter.foundation.sources.count": sources.length
      });
      try {
        const target = await this.runWithNewSpan(
          "loadTarget",
          () => this.loadTarget(source, previous)
        );
        getCurrentSpan()?.setAttributes({
          "com.code-like-a-carpenter.foundation.target.exists": !!target
        });
        const modelToUpdate = await this.runWithNewSpan(
          "update",
          () => this.update(source, sources, target, previous)
        );
        getCurrentSpan()?.setAttributes({
          "com.code-like-a-carpenter.foundation.should-update": !!modelToUpdate
        });
        if (modelToUpdate) {
          await this.runWithNewSpan(
            "updateTargetModel",
            () => updateTargetModel(modelToUpdate)
          );
          return;
        }
      } catch (err) {
        if (err instanceof NotFoundError) {
          const modelToCreate = await this.runWithNewSpan(
            "create",
            () => this.create(source, sources, previous)
          );
          getCurrentSpan()?.setAttributes({
            "com.code-like-a-carpenter.foundation.should-create": !!modelToCreate
          });
          if (modelToCreate) {
            await this.runWithNewSpan(
              "createTargetModel",
              () => createTargetModel(modelToCreate)
            );
          }
          return;
        }
        throw err;
      }
    });
  }
};
export {
  Reducer,
  makeReducer
};
//# sourceMappingURL=make-reducer.mjs.map
