import type { WithExceptionTracing } from '../../dependencies';
import type { ResultType } from '../../types';
import { CDCHandler } from '../common/cdc-handler';
import type { Handler } from '../common/handlers';
import type { UnmarshalledDynamoDBRecord } from '../common/unmarshall-record';
interface SDK<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT> {
    createTargetModel: (target: CREATE_TARGET_INPUT) => Promise<ResultType<TARGET>>;
    unmarshallSourceModel: (item: Record<string, unknown>) => SOURCE;
    updateTargetModel: (target: UPDATE_TARGET_INPUT) => Promise<ResultType<TARGET>>;
}
/**
 * Returns a function that handles a DynamoDB for the SOURCE and produces a
 * TARGET
 */
export declare function makeEnricher<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT>(constructor: EnricherConstructor<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT>, sdk: SDK<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT>, dependencies: WithExceptionTracing): Handler;
export interface EnricherConstructor<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT> {
    new (sdk: SDK<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT>): Enricher<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT>;
}
export declare abstract class Enricher<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT> extends CDCHandler {
    sdk: SDK<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT>;
    constructor(sdk: SDK<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT>);
    enrich(unmarshalledRecord: UnmarshalledDynamoDBRecord): Promise<void>;
    abstract load(record: SOURCE, previous: SOURCE | undefined): Promise<TARGET>;
    abstract create(record: SOURCE, previous: SOURCE | undefined): Promise<CREATE_TARGET_INPUT | undefined>;
    abstract update(record: SOURCE, target: TARGET, previous: SOURCE | undefined): Promise<UPDATE_TARGET_INPUT | undefined>;
}
export {};
//# sourceMappingURL=make-enricher.d.ts.map