import type { WithExceptionTracing } from '../../dependencies';
import type { ResultType } from '../../types';
import { CDCHandler } from '../common/cdc-handler';
import type { Handler } from '../common/handlers';
import type { UnmarshalledDynamoDBRecord } from '../common/unmarshall-record';
interface SDK<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT> {
    createTargetModel: (target: CREATE_TARGET_INPUT) => Promise<ResultType<TARGET>>;
    unmarshallSourceModel: (item: Record<string, unknown>) => SOURCE;
    updateTargetModel: (target: UPDATE_TARGET_INPUT) => Promise<ResultType<TARGET>>;
}
/**
 * Makes an SQS handler that expects the payload to be a DynamoDB Stream Record.
 */
export declare function makeMultiReducer<SOURCE, TARGET, CREATE_TARGET_INPUT extends object, UPDATE_TARGET_INPUT extends object>(constructor: MultiReducerConstructor<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT>, sdk: SDK<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT>, dependencies: WithExceptionTracing): Handler;
export interface MultiReducerConstructor<SOURCE, TARGET, CREATE_TARGET_INPUT extends object, UPDATE_TARGET_INPUT extends object> {
    new (sdk: SDK<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT>): MultiReducer<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT>;
}
export declare abstract class MultiReducer<SOURCE, TARGET, CREATE_TARGET_INPUT extends object, UPDATE_TARGET_INPUT extends object> extends CDCHandler {
    sdk: SDK<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT>;
    constructor(sdk: SDK<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT>);
    reduce(unmarshalledRecord: UnmarshalledDynamoDBRecord): Promise<void>;
    protected abstract loadTargets(source: SOURCE, previous: SOURCE | undefined): Promise<TARGET[]>;
    protected abstract loadSources(source: SOURCE, previous: SOURCE | undefined): Promise<readonly SOURCE[]>;
    protected abstract createOrUpdate(source: SOURCE, sources: readonly SOURCE[], targets: readonly TARGET[], previous: SOURCE | undefined): Promise<(CREATE_TARGET_INPUT | UPDATE_TARGET_INPUT)[]>;
}
export {};
//# sourceMappingURL=make-multi-reducer.d.ts.map